class Endpoints:
      def __init__(self) -> None:
            pass

      @staticmethod
      def get_endpoint(modality: str) -> str:
            if modality == "llm":
                  return "/llm-ai/chat"
            elif modality == "tts":
                  return "/tts-ai/chat"
            elif modality == "stt":
                  return "/stt-ai/chat"
            
      @staticmethod
      def get_endpoint_async(modality:str) -> str:
            if modality == "llm":
                  return "/llm-ai/chat_async"
            elif modality == "tts":
                  return "/tts-ai/chat_async"
            elif modality == "stt":
                  return "/stt-ai/chat_async"
            
      @staticmethod
      def get_file_endpoint(modality: str) -> str:
            if modality == "llm":
                  return ""
            elif modality == "stt":
                  return ""
            elif modality == "tts":
                  return "tts-ai/download"


from pydantic import BaseModel

class OutputAI(BaseModel):
      project_id: str

"""
{
  "project_id": "723f4e98-d616-4ce2-9a1b-2ed4a1bafd17",
  "model_name": "mistral-medium-2505",
  "latency": 1.3263087999075651,
  "input": "selam",
  "reasoning": "stop",
  "output": "Merhaba! 👋 Nasıl yardımcı olabilirim? Bir proje hakkında sorularınız mı var, yoksa kodlama konusunda yardım mı istiyorsunuz? 😊",
  "input_token": 15,
  "cached_token": 0,
  "reasoning_token": 0,
  "output_token": 46,
  "input_cost": 0.00007500000000000001,
  "cached_cost": 0,
  "reasoning_cost": 0,
  "output_cost": 0.00069,
  "total_cost": 0.00069
}
"""

class OutputLLM(OutputAI):
      project_id: str
      model_name: str
      latency: float
      input: str
      reasoning: str
      output: str
      input_token: int
      cached_token: int
      reasoning_token: int
      output_token: int
      input_cost: float
      cached_cost: float
      reasoning_cost: float
      output_cost: float
      total_cost: float

class OutputSTT(OutputAI):
      project_id: str
      model_name: str
      audio_file_minute: int
      text: str
      total_cost: float

class OutputTTS(OutputAI):
      project_id: str
      model_name: str
      file_path: str
      mp3_minute: int
      total_cost: float


# Standard Packages
from typing import Optional
from dataclasses import dataclass
from abc import ABC

# Types
from taylorist.ai.types.io_types import OutputLLM

# Modality Provider-Model Definations
from taylorist.ai.llm.model_defination import LLMModel
from taylorist.ai.llm.provider_defination import LLMProvider

# Server
from taylorist.server.server import TayloristServer

# Converters
from taylorist.converters.converter_input import ConverterJson, Payload

class TayloristLLM:
      def __init__(self, api_key: str):
            self.__server = TayloristServer()
            self.api_key = api_key


      def chat(self, 
               prompt: str,
               model: Optional[LLMModel] = "gpt-3.5-turbo",
               provider: Optional[LLMProvider] = "OpenAI",
               instruction: Optional[str] = "",
               project_id: Optional[str] = None,
               top_p: Optional[float] = 1,
               temperature: Optional[float] = 1,
               max_tokens: Optional[float] = None) -> OutputLLM:
            pass

      async def chat_async(self,
                     prompt: str,
                     model: Optional[LLMModel] = "gpt-4o",
                     provider: Optional[LLMProvider] = "OpenAI",
                     instruction: Optional[str] = "",
                     project_id: Optional[str] = None,
                     top_p: Optional[float] = 1,
                     temperature: Optional[float] = 1,
                     max_tokens: Optional[int] = 1000) -> OutputLLM:
      
            payload = ConverterJson.convert_llm_json(
                  api_key= self.api_key,
                  project_id= project_id,
                  provider= provider,
                  model= model,
                  instruction= instruction,
                  prompt= prompt,
                  top_p= top_p,
                  temperature= temperature,
                  max_tokens= max_tokens
            )

            api : OutputLLM = await self.__server.chat_async(
                  modality= "llm",
                  payload= payload
            )

            return api

async def main():
      llm = TayloristLLM()

      search = await llm.chat_async(prompt= "Bana havayı araştır")


# Standard Packages
from typing import Optional

# Server
from taylorist.server.server import TayloristServer

# Types
from taylorist.ai.types.io_types import OutputSTT

# Converters
from taylorist.converters.converter_input import ConverterJson, Payload, FileDownload

class TayloristSTT:
      def __init__(self, api_key: str) -> None:
            self.api_key = api_key
            self.__server = TayloristServer()

      def chat(self) -> OutputSTT:
            pass

      async def chat_async(self,
                     audio_file_path: str,
                     project_id: Optional[str] = None,
                     provider: Optional[str] = "Deepgram",
                     model: Optional[str] = "suno-3"
                     ) -> OutputSTT:
            
            payload = ConverterJson.convert_stt_json(
                  api_key= self.api_key,
                  project_id= project_id,
                  provider= provider,
                  model= model,
                  audio_file_path= audio_file_path
            )

            api : OutputSTT = await self.__server.chat_async(
                  modality= "stt",
                  payload= payload
            )

            return api


# taylorist/ai/__init__.py

from .tts import TayloristTTS, TTSModel, TTSProvider, TTSVoice
from .stt import TayloristSTT, STTModel, STTProvider
from .llm import TayloristLLM, LLMModel, LLMProvider
from .types import OutputLLM, OutputSTT, OutputTTS

__all__ = [
    # TTS
    "TayloristTTS", "TTSModel", "TTSProvider", "TTSVoice",
    # STT
    "TayloristSTT", "STTModel", "STTProvider",
    # LLM
    "TayloristLLM", "LLMModel", "LLMProvider",
    # Types
    "OutputLLM", "OutputSTT", "OutputTTS",
]



import traceback
import sys

class TayloristException(Exception):
    def __init__(self, *args: object, status_code: int, message: str) -> None:
        super().__init__(*args)
        self.status_code = status_code
        self.message = message

        print(f"[TayloristException] {self.message} (status_code={self.status_code})")
        traceback.print_stack()

class TayloristAPIException(Exception):
    def __init__(self, *args: object, status_code: int, message: str) -> None:
        super().__init__(*args)
        self.status_code = status_code
        self.message = message

        print(f"[TayloristAPIException] {self.message} (status_code={self.status_code})")
        traceback.print_stack()



# taylorist/__init__.py

from taylorist.ai import (
    TayloristTTS, TTSModel, TTSProvider, TTSVoice,
    TayloristSTT, STTModel, STTProvider,
    TayloristLLM, LLMModel, LLMProvider,
    OutputLLM, OutputSTT, OutputTTS,
)

__all__ = [
    # TTS
    "TayloristTTS", "TTSModel", "TTSProvider", "TTSVoice",
    # STT
    "TayloristSTT", "STTModel", "STTProvider",
    # LLM
    "TayloristLLM", "LLMModel", "LLMProvider",
    # Types
    "OutputLLM", "OutputSTT", "OutputTTS",
]



from taylorist.ai.llm.taylorist_llm import TayloristLLM
from taylorist.ai.llm.model_defination import LLMModel
from taylorist.ai.llm.provider_defination import LLMProvider

__all__ = ["TayloristLLM", "LLMModel", "LLMProvider"]


from typing import Union, Optional
from dataclasses import dataclass

@dataclass
class FileDownload:
      file_download: Optional[bool]
      filepath: Optional[str]

@dataclass
class Payload:
      json_data: Optional[dict]
      form_data: Optional[dict]
      file_data: Optional[dict]
      output_download: Optional[FileDownload]

class ConverterJson:

      @staticmethod
      def convert_llm_json(
            api_key: str,
            project_id: str,
            provider: str,
            model: str,
            instruction: str,
            prompt: str,
            top_p: float,
            temperature: float,
            max_tokens: int) -> Payload:
            
            return Payload(
                  json_data= {
                        "api_key": api_key,
                        "project_id": project_id,
                        "provider": provider,
                        "model": model,
                        "instruction": instruction,
                        "prompt": prompt,
                        "temperature": temperature,
                        "top_p": top_p,
                        "max_tokens": max_tokens
                  },
                  form_data= None,
                  file_data= None,
                  output_download= None
            )
      
      @staticmethod
      def convert_stt_json(
            api_key: str,
            audio_file_path: str,
            project_id: str,
            provider: str,
            model: str
      ) -> Payload:
            return Payload(
                  json_data= {
                  },
                  form_data = {
                        "api_key": api_key,
                        "project_id": project_id,
                        "provider": provider,
                        "model": model
                  },
                  file_data= {
                        "audio_file": open(audio_file_path, "rb")
                  },
                  output_download= FileDownload(
                        file_download= False,
                        filepath= None
                  )
            )
            

      @staticmethod
      def convert_tts_json(
            api_key: str,
            project_id: str,
            provider: str,
            model: str,
            instruction: str,
            prompt: str,
            voice: str
      ) -> Payload:
            return Payload(
                  json_data={
                        "api_key": api_key,
                        "project_id": project_id,
                        "provider": provider,
                        "model": model,
                        "voice": voice,
                        "instruction": instruction,
                        "prompt": prompt
                  },
                  form_data= None,
                  file_data= None,
                  output_download= FileDownload(
                        file_download= True,
                        filepath= ""
                  )
            )


# io
from taylorist.ai.types.io_types import OutputAI, OutputLLM, OutputSTT, OutputTTS

class ConverterOutputType:

      @staticmethod
      def convert_ai_output(modality: str, api_response) -> OutputAI:
            match modality:
                  case "llm":
                        return ConverterOutputType.convert_to_output_llm(api_response)
                  case "tts":
                        return ConverterOutputType.convert_to_output_tts(api_response)
                  case "stt":
                        return ConverterOutputType.convert_to_output_stt(api_response)
                  
      @staticmethod
      def convert_to_output_llm(api_response) -> OutputLLM:
            return OutputLLM(
                  project_id= api_response["project_id"],
                  model_name= api_response["model_name"],
                  latency= api_response["latency"],
                  input= api_response["input"],
                  reasoning= api_response["reasoning"],
                  output= api_response["output"],
                  input_token= api_response["input_token"],
                  cached_token= api_response["cached_token"],
                  reasoning_token= api_response["reasoning_token"],
                  output_token= api_response["output_token"],
                  input_cost= api_response["input_cost"],
                  cached_cost= api_response["cached_cost"],
                  reasoning_cost= api_response["reasoning_cost"],
                  output_cost= api_response["output_cost"],
                  total_cost= api_response["total_cost"]
            )

      @staticmethod
      def convert_to_output_stt(api_response) -> OutputSTT:
            return OutputSTT(
                  project_id= api_response["project_id"],
                  model_name= api_response["model"],
                  audio_file_minute= api_response["audio_file_minute"],
                  text= api_response["text"],
                  total_cost= api_response["total_cost"]
            )

      @staticmethod
      def convert_to_output_tts(api_response) -> OutputTTS:
            return OutputTTS(
                  project_id= api_response["project_id"],
                  model_name= api_response["model_name"],
                  file_path= api_response["file_path"],
                  mp3_minute= api_response["mp3_minutes"],
                  total_cost= api_response["total_cost"]
            )


# Standard Packages
from pathlib import Path
from typing import Optional

# Modality Definations - Provider | Model
from taylorist.ai.tts.provider_defination import TTSProvider
from taylorist.ai.tts.model_defination import TTSModel

# I/O Types
from taylorist.ai.types.io_types import OutputTTS

# Server
from taylorist.server.server import TayloristServer

# Converters
from taylorist.converters.converter_input import ConverterJson, Payload, FileDownload

class TayloristTTS:
      def __init__(self, api_key: str) -> None:
            self.__server = TayloristServer()
            self.api_key = api_key

      def chat(self,
               prompt: str,
               instruction: Optional[str] = None,
               voice: Optional[str] = None,
               model: Optional[TTSModel] = "tts-1",
               provider: Optional[TTSProvider] = "OpenAI",
               project_id: Optional[str] = None) -> OutputTTS:
            pass


      async def chat_async(self,
                           prompt: str,
                           instruction: Optional[str] = None,
                           voice: Optional[str] = None,
                           model: Optional[TTSModel] = "tts-1",
                           provider: Optional[TTSProvider] = "OpenAI",
                           project_id: Optional[str] = None) -> OutputTTS:
            
            """create-payload"""
            payload = ConverterJson.convert_tts_json(
                  api_key= self.api_key,
                  project_id= project_id,
                  provider= provider,
                  model= model,
                  instruction= instruction,
                  prompt = prompt,
                  voice= voice
            )
            
            """api request"""
            api : OutputTTS = await self.__server.chat_async(
                  modality= "tts",
                  payload= payload,
            )

            filename = Path(api.file_path).name

            """file download"""
            file_download = await self.__server.file_download(
                  modality="tts",
                  file= FileDownload(
                        file_download= True,
                        filepath= filename
                  )
            )
            api.file_path = str(file_download)

            """returned"""    
            return api


# Standard Package
import httpx
import aiofiles
from uuid import uuid4

# Taylorist Exceptions
from taylorist.errors.exception import TayloristException, TayloristAPIException

# Type
from taylorist.ai.types.io_types import OutputAI

# Helpers
from .endpoints import Endpoints
from ..converters.converter_output import ConverterOutputType
from ..converters.converter_input import Payload, ConverterJson, FileDownload
from pathlib import Path

class TayloristServer:
      def __init__(self) -> None:
            self.base_url = "https://taylorist-server-production.up.railway.app"
            self.project_root = Path(__file__).resolve().parents[2]
            self.download_root = self.project_root / "speech"     

      def chat(self, input) -> OutputAI:
            pass

      async def chat_async(self, modality: str, payload: Payload) -> OutputAI:
            try:
                  url = f"{self.base_url}{Endpoints.get_endpoint_async(modality)}"

                  timeout = httpx.Timeout(30.0)

                  async with httpx.AsyncClient(timeout= timeout) as client:
                        response = await client.post(url, json= payload.json_data, data=payload.form_data, files= payload.file_data)

                        if response.status_code == 200:
                              response_data = response.json()

                              return ConverterOutputType.convert_ai_output(modality= modality, api_response= response_data)
                        else:
                              raise TayloristAPIException(status_code= response.status_code, message= str(response))

            except httpx.HTTPError as ex:
                  raise TayloristException(status_code= 500, message=str(ex))
            
      async def file_download(self, modality: str, file: FileDownload):
            if not file.filepath:
                  raise TayloristException(400, "filepath parametresi boş")

            url = f"{self.base_url}/{Endpoints.get_file_endpoint(modality)}/{file.filepath}"

            async with httpx.AsyncClient() as client, \
                        client.stream("GET", url) as resp:
                  resp.raise_for_status()

                  target_dir = self.download_root / modality      
                  target_dir.mkdir(parents=True, exist_ok=True)
                  target_file = target_dir / f"{uuid4()}.mp3"

                  total = 0
                  async with aiofiles.open(target_file, "wb") as out_f:
                        async for chunk in resp.aiter_bytes():
                              if chunk:
                                    await out_f.write(chunk)
                                    total += len(chunk)

                  return target_file


from typing import Literal

LLMProvider = Literal[
      "OpenAI",
      "Anthropic",
      "Google",
      "xAI",
      "Qwen",
      "Deepseek",
      "Mistral",
      "Cohere",
      "HuggingFace"
]


from taylorist.ai.stt.taylorist_stt import TayloristSTT
from taylorist.ai.stt.model_defination import STTModel
from taylorist.ai.stt.provider_defination import STTProvider

__all__ = ["TayloristSTT", "STTModel", "STTProvider"]


from setuptools import setup, find_packages

setup(
      name="taylorist",
      version="0.1.0",
      packages=find_packages(),
      include_package_data=True,
      package_data={
          "taylorist": ["**/*.py", "**/py.typed"],
      },
      author="umutkrts",
      long_description=open("README.md").read(),
      long_description_content_type="text/markdown",
      url="https://github.com/umutkrts07/taylorist",
      project_urls={
          "Bug Tracker": "https://github.com/umutkrts07/taylorist/issues",
          "Source Code": "https://github.com/umutkrts07/taylorist",
      },
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
      ],
    install_requires=[
        "httpx>=0.28.1",
        "requests>=2.32.3",
        "pydantic>=2.11.5",
        "aiohttp>=3.12.7"
    ],
    python_requires=">=3.8",
)


from taylorist.ai.tts.taylorist_tts import TayloristTTS
from taylorist.ai.tts.model_defination import TTSModel
from taylorist.ai.tts.provider_defination import TTSProvider
from taylorist.ai.tts.voice_defination import TTSVoice

__all__ = ["TayloristTTS", "TTSModel", "TTSProvider", "TTSVoice"]